# 安装服务登记

## 使用 Helm 安装 服务登记

服务登记（Service Catalog）是一种扩展 API，它能让 Kubernetes 集群中运行的应用易于使用外部托管的的软件服务，例如云供应商提供的数据仓库服务。

服务登记可以检索、供应、和绑定由 [服务代理人（Service Brokers）](https://v1-14.docs.kubernetes.io/zh/docs/reference/glossary/?all=true#term-service-broker) 提供的外部 [托管服务](https://v1-14.docs.kubernetes.io/zh/docs/reference/glossary/?all=true#term-managed-service)，而无需知道那些服务具体是怎样创建和托管的。

使用 [Helm](https://helm.sh/) 在 Kubernetes 集群上安装 服务登记。 要获取有关此过程的最新信息，请浏览 [kubernetes-incubator/service-catalog](https://github.com/kubernetes-incubator/service-catalog/blob/master/docs/install.md) 仓库。

### 准备开始

-   理解 [Service Catalog](https://v1-14.docs.kubernetes.io/docs/concepts/service-catalog/) 的关键概念。

-   服务登记 需要 Kubernetes 集群版本在 1.7 或更高版本。

-   您必须启用 Kubernetes 集群的 DNS 功能。

    -   如果使用基于云的 Kubernetes 集群或 

        

    [Minikube](https://v1-14.docs.kubernetes.io/docs/getting-started-guides/minikube/)，则可能已经启用了集群 DNS。 * 如果您正在使用 `hack/local-up-cluster.sh`，请确保设置了 `KUBE_ENABLE_CLUSTER_DNS` 环境变量，然后运行安装脚本。

-   [安装和设置 v1.7 或更高版本的 kubectl](https://kubernetes.io/docs/tasks/tools/install-kubectl/)，确保将其配置为连接到 Kubernetes 集群。

-   安装 v2.7.0 或更高版本的 [Helm](http://helm.sh/)。

    -   遵照 [Helm 安装说明](https://github.com/kubernetes/helm/blob/master/docs/install.md)。
    -   如果已经安装了适当版本的 Helm，请执行 `helm init` 来安装 Helm 的服务器端组件 Tiller。

### 添加 service-catalog Helm 仓库

安装 Helm 后，通过执行以下命令将 *service-catalog* Helm 存储库添加到本地计算机：

```shell
helm repo add svc-cat https://svc-catalog-charts.storage.googleapis.com
```

通过执行以下命令进行检查，以确保安装成功：

```shell
helm search service-catalog
```

如果安装成功，该命令应输出以下内容：

```
NAME            VERSION DESCRIPTION
svc-cat/catalog 0.0.1   service-catalog API server and controller-manag...
```

### 启用 RBAC

您的 Kubernetes 集群必须启用 RBAC，这需要您的 Tiller Pod 具有 `cluster-admin` 访问权限。

如果您使用的是 Minikube，请使用以下参数运行 `minikube start` 命令：

```shell
minikube start --extra-config=apiserver.Authorization.Mode=RBAC
```

如果您使用 `hack/local-up-cluster.sh`，请使用以下值设置 `AUTHORIZATION_MODE` 环境变量：

```
AUTHORIZATION_MODE=Node,RBAC hack/local-up-cluster.sh -O
```

默认情况下，`helm init` 将 Tiller Pod 安装到 `kube-system` 命名空间，Tiller 配置为使用 `default` 服务帐户。

>   Note:
>
>   如果在运行 `helm init` 时使用了 `--tiller-namespace` 或 `--service-account` 参数，则需要调整以下命令中的 `--serviceaccount` 参数以引用相应的 namespace 和 ServiceAccount 名称。

配置 Tiller 以获得 `cluster-admin` 访问权限：

```shell
kubectl create clusterrolebinding tiller-cluster-admin \
    --clusterrole=cluster-admin \
    --serviceaccount=kube-system:default
```

### 在 Kubernetes 集群中安装 服务登记

使用以下命令从 Helm 存储库的根目录安装 Service 服务登记：

```shell
helm install svc-cat/catalog \
    --name catalog --namespace catalog
```

### 接下来

-   查看[示例服务代理](https://github.com/openservicebrokerapi/servicebroker/blob/mastergettingStarted.md#sample-service-brokers)。
-   探索 [kubernetes-incubator/service-catalog](https://github.com/kubernetes-incubator/service-catalog) 项目。











## 使用 SC 安装服务登记

服务登记（Service Catalog）是一种扩展 API，它能让 Kubernetes 集群中运行的应用易于使用外部托管的的软件服务，例如云供应商提供的数据仓库服务。



服务登记可以检索、供应、和绑定由 [服务代理人（Service Brokers）](https://v1-14.docs.kubernetes.io/zh/docs/reference/glossary/?all=true#term-service-broker) 提供的外部 [托管服务](https://v1-14.docs.kubernetes.io/zh/docs/reference/glossary/?all=true#term-managed-service)，而无需知道那些服务具体是怎样创建和托管的。

使用[服务登记安装程序](https://github.com/GoogleCloudPlatform/k8s-service-catalog#installation)工具可以轻松地在 Kubernetes 集群上安装或卸载服务登记。 这个 CLI 工具以 `sc` 命令形式被安装在您的本地环境中。

### 准备开始

-   了解[服务登记](https://v1-14.docs.kubernetes.io/docs/concepts/service-catalog/)的主要概念。

-   安装 [Go 1.6+](https://golang.org/dl/) 以及设置 `GOPATH`。

-   安装生成 SSL 工件所需的 [cfssl](https://github.com/cloudflare/cfssl) 工具。

-   服务登记需要 Kubernetes 1.7+ 版本。

-   [安装和设置 kubectl](https://v1-14.docs.kubernetes.io/docs/tasks/tools/install-kubectl/)，以便将其配置为连接到 Kubernetes v1.7+ 集群。

-   要安装服务登记，kubectl 用户必须绑定到 *cluster-admin* 角色。为了确保这是正确的，请运行以下命令：

    ```
    kubectl create clusterrolebinding cluster-admin-binding --clusterrole=cluster-admin --user=<user-name>
    ```

### 在本地环境中安装 `sc`

使用 `go get` 命令安装 `sc` CLI 工具：

```Go
go get github.com/GoogleCloudPlatform/k8s-service-catalog/installer/cmd/sc
```

执行上述命令后，`sc` 应被安装在 `GOPATH/bin` 目录中了。

### 在 Kubernetes 集群中安装服务登记

首先，检查是否已经安装了所有依赖项。运行：

```shell
sc check
```

如检查通过，应输出：

```
Dependency check passed. You are good to go.
```

接下来，运行安装命令并指定要用于备份的 `storageclass`：

```shell
sc install --etcd-backup-storageclass "standard"
```

### 卸载服务登记

如果您想使用 `sc` 工具从 Kubernetes 集群卸载服务登记，请运行：

```shell
sc uninstall
```

### 接下来

-   查看 [服务代理示例](https://github.com/openservicebrokerapi/servicebroker/blob/master/gettingStarted.md#sample-service-brokers)。
-   探索 [kubernetes-incubator/service-catalog](https://github.com/kubernetes-incubator/service-catalog) 项目。