# 管理集群的守护进程

## 对守护进程执行滚动更新

本文介绍了如何对 DaemonSet 执行滚动更新。

### 准备开始

-   Kubernetes 1.6 或者更高版本中才支持 DaemonSet 滚动更新功能。

### DaemonSet 更新策略

DaemonSet 有两种更新策略：

-   OnDelete: 使用 `OnDelete` 更新策略时，在更新 DaemonSet 模板后，只有当您手动删除老的 DaemonSet pods 之后，新的 DaemonSet pods *才会*被自动创建。跟 Kubernetes 1.6 以前的版本类似。
-   RollingUpdate: 这是默认的更新策略。使用 `RollingUpdate` 更新策略时，在更新 DaemonSet 模板后，老的 DaemonSet pods 将被终止，并且将以受控方式自动创建新的 DaemonSet pods。

### 执行滚动更新

要启用 DaemonSet 的滚动更新功能，必须设置 `.spec.updateStrategy.type` 为 `RollingUpdate`。

您可能想设置[`.spec.updateStrategy.rollingUpdate.maxUnavailable`](https://v1-14.docs.kubernetes.io/docs/concepts/workloads/controllers/deployment/#max-unavailable) (默认为 1) 和[`.spec.minReadySeconds`](https://v1-14.docs.kubernetes.io/docs/concepts/workloads/controllers/deployment/#min-ready-seconds) (默认为 0)。

#### 步骤 1: 检查 DaemonSet 的滚动更新策略

首先，检查 DaemonSet 的更新策略，确保已经将其设置为 `RollingUpdate`:

```shell
kubectl get ds/<daemonset-name> -o go-template='{{.spec.updateStrategy.type}}{{"\n"}}'
```

如果还没在系统中创建 DaemonSet，请使用以下命令检查 DaemonSet 的清单：

```shell
kubectl create -f ds.yaml --dry-run -o go-template='{{.spec.updateStrategy.type}}{{"\n"}}'
```

两个命令的输出都应该为：

```shell
RollingUpdate
```

如果输出不是 `RollingUpdate`，请返回并相应地修改 DaemonSet 对象或者清单。

#### 步骤 2：使用 `RollingUpdate` 更新策略创建 DaemonSet

如果已经创建了 DaemonSet，则可以跳过该步骤并跳转到步骤 3。

验证 DaemonSet 清单的更新策略后，创建 DaemonSet：

```shell
kubectl create -f ds.yaml
```

或者，您打算使用 `kubectl apply` 更新 DaemonSet，请使用 `kubectl apply` 创建相同的 DaemonSet。

```shell
kubectl apply -f ds.yaml
```

#### 步骤 3：更新 DaemonSet 模板

对 `RollingUpdate` DaemonSet `.spec.template` 的任何更新都将触发滚动更新。这可以通过几个不同的 `kubectl` 命令来完成。

##### 声明式命令

如果您使用[配置文件](https://v1-14.docs.kubernetes.io/docs/concepts/overview/object-management-kubectl/declarative-config/)来更新 DaemonSets，请使用 `kubectl apply`:

```shell
kubectl apply -f ds-v2.yaml
```

##### 命令式命令

如果您使用[命令式命令](https://v1-14.docs.kubernetes.io/docs/concepts/overview/object-management-kubectl/imperative-command/)来更新 DaemonSets，请使用`kubectl edit` 或者 `kubectl patch`:

```shell
kubectl edit ds/<daemonset-name>
kubectl patch ds/<daemonset-name> -p=<strategic-merge-patch>
```

###### 只更新容器镜像

如果您只需要更新 DaemonSet 模板里的容器镜像，比如，`.spec.template.spec.containers[*].image`, 请使用 `kubectl set image`:

```shell
kubectl set image ds/<daemonset-name> <container-name>=<container-new-image>
```

#### 步骤 4：查看滚动更新状态

最后，观察 DaemonSet 最新滚动更新的进度：

```shell
kubectl rollout status ds/<daemonset-name>
```

当滚动更新完成时，输出结果如下：

```shell
daemonset "<daemonset-name>" successfully rolled out
```

### 故障排查

#### DaemonSet 滚动更新卡住

有时，DaemonSet 滚动更新可能会卡住。可能原因如下：

##### 一些节点资源用尽

由于新 DaemonSet pods 无法调度到至少一个节点时，滚动更新就会卡住。这可能是由于节点已经[资源用尽](https://v1-14.docs.kubernetes.io/docs/tasks/administer-cluster/out-of-resource/)。

发生这种情况时，通过对 `kubectl get nodes` 和下面命令行的输出作比较，找出没有调度部署 DaemonSet pods 的节点：

```shell
kubectl get pods -l <daemonset-selector-key>=<daemonset-selector-value> -o wide
```

一旦找到这些节点，从节点上删除一些非 DaemonSet pods，为新的 DaemonSet pods 腾出空间。

>   **Note:** 当所删除的 pods 不受任何控制器管理，也不是多副本的 pods，上述操作将导致服务中断。 同时，上述操作也不会考虑 [PodDisruptionBudget](https://v1-14.docs.kubernetes.io/docs/tasks/configure-pod-container/configure-pod-disruption-budget/) 所施加的约束。

##### 滚动中断

如果最近的 DaemonSet 模板更新被破坏了，比如，容器处于崩溃循环状态或者容器镜像不存在(通常由于拼写错误)，就会发生 DaemonSet 滚动更新中断。

要解决此问题，只需再次更新 DaemonSet 模板即可。以前不健康的滚动更新不会阻止新的滚动更新。

##### 时钟偏差

如果在 DaemonSet 中指定了 `.spec.minReadySeconds`，主节点和工作节点之间的时钟偏差会使 DaemonSet 无法检测到正确的滚动更新进度。

### 接下来

-   查看[任务: 在 DaemonSet 上执行回滚](https://v1-14.docs.kubernetes.io/docs/tasks/manage-daemon/rollback-daemon-set/)
-   查看[概念: 创建 DaemonSet 以适应现有的 DaemonSet pods](https://v1-14.docs.kubernetes.io/docs/concepts/workloads/controllers/daemonset/)





## 对守护进程执行回滚

本文展示了如何对 DaemonSet 执行回滚。

### 准备工作

-   DaemonSet 滚动升级历史和 DaemonSet 回滚特性仅在 Kubernetes 1.7 及以后版本的 `kubectl` 中支持。
-   确保您了解如何 [对 DaemonSet 执行滚动升级](https://kubernetes.io/docs/tasks/manage-daemon/update-daemon-set/)。

### 对 DaemonSet 执行回滚

步骤 1： 找到想要 DaemonSet 回滚到的历史版本（revision）

如果只想回滚到最后一个版本，可以跳过这一步。

列出 DaemonSet 的所有版本：

```shell
kubectl rollout history daemonset <daemonset-name>
```

该命令返回 DaemonSet 版本列表：

```shell
daemonsets "<daemonset-name>"
REVISION        CHANGE-CAUSE
1               ...
2               ...
...
```

-   在创建时，DaemonSet 的变化原因从 `kubernetes.io/change-cause` 注解（annotation）复制到其版本中。 用户可以在 `kubectl` 中指定 `--record=true` ，将执行的命令记录在变化原因注解中。

执行以下命令，来查看指定版本的详细信息：

```shell
kubectl rollout history daemonset <daemonset-name> --revision=1
```

该命令返回相应版本的详细信息：

```shell
daemonsets "<daemonset-name>" with revision #1
Pod Template:
Labels:       foo=bar
Containers:
app:
 Image:       ...
 Port:        ...
 Environment: ...
 Mounts:      ...
Volumes:       ...
```

#### 步骤 2： 回滚到指定版本

```shell
# 在 --to-revision 中指定您从步骤 1 中获取的版本序号
kubectl rollout undo daemonset <daemonset-name> --to-revision=<revision>
```

如果成功，命令会返回：

```shell
daemonset "<daemonset-name>" rolled back
```

如果 `--to-revision` 参数未指定，将选中最近的版本。

#### 步骤 3： 观察 DaemonSet 回滚进度

`kubectl rollout undo daemonset` 向服务器表明启动 DaemonSet 回滚。 真正的回滚是在服务器端异步完成的。

执行以下命令，来观察 DaemonSet 回滚进度：

```shell
kubectl rollout status ds/<daemonset-name> 
```

回滚完成时，输出形如：

```shell
daemonset "<daemonset-name>" successfully rolled out
```

### 理解 DaemonSet 版本

在前面的 `kubectl rollout history` 步骤中，您获得了一个版本列表，每个版本都存储在名为 `ControllerRevision` 的资源中。 `ControllerRevision` 仅在 Kubernetes 1.7 及以后的版本中可用。

查找原始的版本资源，来查看每个版本中存储了什么内容：

```shell
kubectl get controllerrevision -l <daemonset-selector-key>=<daemonset-selector-value>
```

该命令返回 `ControllerRevisions` 列表：

```shell
NAME                               CONTROLLER                     REVISION   AGE
<daemonset-name>-<revision-hash>   DaemonSet/<daemonset-name>     1          1h
<daemonset-name>-<revision-hash>   DaemonSet/<daemonset-name>     2          1h
```

每个 `ControllerRevision` 中存储了相应 DaemonSet 版本的注解和模板。

`kubectl rollout undo` 采用特定 `ControllerRevision` ，并用 `ControllerRevision` 中存储的模板代替 DaemonSet 的模板。 `kubectl rollout undo` 相当于通过其他命令（如 `kubectl edit` 或 `kubectl apply`）将 DaemonSet 模板更新至先前的版本。

注意 DaemonSet 版本只会向前滚动。 也就是说，回滚完成后，所回滚到的 `ControllerRevision` 版本号 (`.revision` 字段) 会增加。 例如，如果用户在系统中有版本 1 和版本 2，并从版本 2 回滚到版本 1 ，带有 `.revision: 1` 的`ControllerRevision` 将变为 `.revision: 3`。

### 故障排除

-   查看 [DaemonSet 滚动升级故障排除](https://kubernetes.io/docs/tasks/manage-daemon/update-daemon-set/#troubleshooting)。